# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven, SequenceNotStr
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.boxes import proxy_set_params
from ....types.v1.boxes.proxy_get_response import ProxyGetResponse
from ....types.v1.boxes.proxy_set_response import ProxySetResponse

__all__ = ["ProxyResource", "AsyncProxyResource"]


class ProxyResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ProxyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return ProxyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProxyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return ProxyResourceWithStreamingResponse(self)

    def clear(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Clear the proxy for the box

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/boxes/{box_id}/proxy",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProxyGetResponse:
        """
        Get the proxy for the box

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._get(
            f"/boxes/{box_id}/proxy",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxyGetResponse,
        )

    def set(
        self,
        box_id: str,
        *,
        host: str,
        port: float,
        auth: proxy_set_params.Auth | NotGiven = NOT_GIVEN,
        excludes: SequenceNotStr[str] | NotGiven = NOT_GIVEN,
        pac_url: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProxySetResponse:
        """
        Set the proxy for the box

        Args:
          host: The host address of the proxy server

          port: The port number of the proxy server

          auth: Box Proxy Auth

          excludes: List of IP addresses and domains that should bypass the proxy. These addresses
              will be accessed directly without going through the proxy server. Default is
              ['127.0.0.1', 'localhost']

          pac_url: PAC (Proxy Auto-Configuration) URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._post(
            f"/boxes/{box_id}/proxy",
            body=maybe_transform(
                {
                    "host": host,
                    "port": port,
                    "auth": auth,
                    "excludes": excludes,
                    "pac_url": pac_url,
                },
                proxy_set_params.ProxySetParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxySetResponse,
        )


class AsyncProxyResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncProxyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return AsyncProxyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProxyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return AsyncProxyResourceWithStreamingResponse(self)

    async def clear(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Clear the proxy for the box

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/boxes/{box_id}/proxy",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProxyGetResponse:
        """
        Get the proxy for the box

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._get(
            f"/boxes/{box_id}/proxy",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxyGetResponse,
        )

    async def set(
        self,
        box_id: str,
        *,
        host: str,
        port: float,
        auth: proxy_set_params.Auth | NotGiven = NOT_GIVEN,
        excludes: SequenceNotStr[str] | NotGiven = NOT_GIVEN,
        pac_url: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProxySetResponse:
        """
        Set the proxy for the box

        Args:
          host: The host address of the proxy server

          port: The port number of the proxy server

          auth: Box Proxy Auth

          excludes: List of IP addresses and domains that should bypass the proxy. These addresses
              will be accessed directly without going through the proxy server. Default is
              ['127.0.0.1', 'localhost']

          pac_url: PAC (Proxy Auto-Configuration) URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._post(
            f"/boxes/{box_id}/proxy",
            body=await async_maybe_transform(
                {
                    "host": host,
                    "port": port,
                    "auth": auth,
                    "excludes": excludes,
                    "pac_url": pac_url,
                },
                proxy_set_params.ProxySetParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProxySetResponse,
        )


class ProxyResourceWithRawResponse:
    def __init__(self, proxy: ProxyResource) -> None:
        self._proxy = proxy

        self.clear = to_raw_response_wrapper(
            proxy.clear,
        )
        self.get = to_raw_response_wrapper(
            proxy.get,
        )
        self.set = to_raw_response_wrapper(
            proxy.set,
        )


class AsyncProxyResourceWithRawResponse:
    def __init__(self, proxy: AsyncProxyResource) -> None:
        self._proxy = proxy

        self.clear = async_to_raw_response_wrapper(
            proxy.clear,
        )
        self.get = async_to_raw_response_wrapper(
            proxy.get,
        )
        self.set = async_to_raw_response_wrapper(
            proxy.set,
        )


class ProxyResourceWithStreamingResponse:
    def __init__(self, proxy: ProxyResource) -> None:
        self._proxy = proxy

        self.clear = to_streamed_response_wrapper(
            proxy.clear,
        )
        self.get = to_streamed_response_wrapper(
            proxy.get,
        )
        self.set = to_streamed_response_wrapper(
            proxy.set,
        )


class AsyncProxyResourceWithStreamingResponse:
    def __init__(self, proxy: AsyncProxyResource) -> None:
        self._proxy = proxy

        self.clear = async_to_streamed_response_wrapper(
            proxy.clear,
        )
        self.get = async_to_streamed_response_wrapper(
            proxy.get,
        )
        self.set = async_to_streamed_response_wrapper(
            proxy.set,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union
from typing_extensions import TypeAlias

from ...._models import BaseModel
from .media_photo import MediaPhoto
from .media_video import MediaVideo

__all__ = ["MediaListMediaResponse", "Data"]

Data: TypeAlias = Union[MediaPhoto, MediaVideo]


class MediaListMediaResponse(BaseModel):
    data: List[Data]
    """List of media files (photos and videos) in the album"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["BrowserOpenTabResponse"]


class BrowserOpenTabResponse(BaseModel):
    id: str
    """The tab id"""

    active: bool
    """Whether the tab is the current active (frontmost) tab"""

    favicon: str
    """The tab favicon"""

    loading: bool
    """Whether the tab is currently in a loading state.

    The value is **true** while the browser is still navigating to the target URL or
    fetching sub-resources (i.e. `document.readyState` is not "complete"). It
    typically switches to **false** once the `load` event fires and all major
    network activity has settled.
    """

    title: str
    """The tab title"""

    url: str
    """The tab url"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from gbox_sdk import GboxClient, AsyncGboxClient
from tests.utils import assert_matches_type
from gbox_sdk.types.v1 import (
    LinuxBox,
    AndroidBox,
    BoxListResponse,
    BoxStopResponse,
    BoxStartResponse,
    BoxDisplayResponse,
    BoxRunCodeResponse,
    BoxRetrieveResponse,
    BoxLiveViewURLResponse,
    BoxWebsocketURLResponse,
    BoxWebTerminalURLResponse,
    BoxExecuteCommandsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBoxes:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: GboxClient) -> None:
        box = client.v1.boxes.retrieve(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(BoxRetrieveResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: GboxClient) -> None:
        response = client.v1.boxes.with_raw_response.retrieve(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = response.parse()
        assert_matches_type(BoxRetrieveResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: GboxClient) -> None:
        with client.v1.boxes.with_streaming_response.retrieve(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = response.parse()
            assert_matches_type(BoxRetrieveResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: GboxClient) -> None:
        box = client.v1.boxes.list()
        assert_matches_type(BoxListResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: GboxClient) -> None:
        box = client.v1.boxes.list(
            device_type="virtual",
            labels={},
            page=1,
            page_size=10,
            status=["running"],
            type=["linux"],
        )
        assert_matches_type(BoxListResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: GboxClient) -> None:
        response = client.v1.boxes.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = response.parse()
        assert_matches_type(BoxListResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: GboxClient) -> None:
        with client.v1.boxes.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = response.parse()
            assert_matches_type(BoxListResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_android(self, client: GboxClient) -> None:
        box = client.v1.boxes.create_android()
        assert_matches_type(AndroidBox, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_android_with_all_params(self, client: GboxClient) -> None:
        box = client.v1.boxes.create_android(
            config={
                "device_type": "virtual",
                "envs": {
                    "ANDROID_LOG_TAGS": "*:V",
                    "ADB_TRACE": "all",
                },
                "expires_in": "15m",
                "labels": {
                    "app": "mobile-testing",
                    "version": "v1.0",
                },
            },
            api_timeout="30s",
            wait=True,
        )
        assert_matches_type(AndroidBox, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create_android(self, client: GboxClient) -> None:
        response = client.v1.boxes.with_raw_response.create_android()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = response.parse()
        assert_matches_type(AndroidBox, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create_android(self, client: GboxClient) -> None:
        with client.v1.boxes.with_streaming_response.create_android() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = response.parse()
            assert_matches_type(AndroidBox, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_linux(self, client: GboxClient) -> None:
        box = client.v1.boxes.create_linux()
        assert_matches_type(LinuxBox, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_linux_with_all_params(self, client: GboxClient) -> None:
        box = client.v1.boxes.create_linux(
            config={
                "envs": {
                    "DEBUG": "true",
                    "API_URL": "https://api.example.com",
                },
                "expires_in": "60m",
                "labels": {
                    "project": "web-automation",
                    "environment": "testing",
                },
            },
            api_timeout="30s",
            wait=True,
        )
        assert_matches_type(LinuxBox, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create_linux(self, client: GboxClient) -> None:
        response = client.v1.boxes.with_raw_response.create_linux()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = response.parse()
        assert_matches_type(LinuxBox, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create_linux(self, client: GboxClient) -> None:
        with client.v1.boxes.with_streaming_response.create_linux() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = response.parse()
            assert_matches_type(LinuxBox, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_display(self, client: GboxClient) -> None:
        box = client.v1.boxes.display(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(BoxDisplayResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_display(self, client: GboxClient) -> None:
        response = client.v1.boxes.with_raw_response.display(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = response.parse()
        assert_matches_type(BoxDisplayResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_display(self, client: GboxClient) -> None:
        with client.v1.boxes.with_streaming_response.display(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = response.parse()
            assert_matches_type(BoxDisplayResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_display(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.with_raw_response.display(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_execute_commands(self, client: GboxClient) -> None:
        box = client.v1.boxes.execute_commands(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            command="ls -l",
        )
        assert_matches_type(BoxExecuteCommandsResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_execute_commands_with_all_params(self, client: GboxClient) -> None:
        box = client.v1.boxes.execute_commands(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            command="ls -l",
            envs={
                "PATH": "/usr/bin:/bin",
                "NODE_ENV": "production",
            },
            api_timeout="30s",
            working_dir="/home/user/projects",
        )
        assert_matches_type(BoxExecuteCommandsResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_execute_commands(self, client: GboxClient) -> None:
        response = client.v1.boxes.with_raw_response.execute_commands(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            command="ls -l",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = response.parse()
        assert_matches_type(BoxExecuteCommandsResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_execute_commands(self, client: GboxClient) -> None:
        with client.v1.boxes.with_streaming_response.execute_commands(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            command="ls -l",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = response.parse()
            assert_matches_type(BoxExecuteCommandsResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_execute_commands(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.with_raw_response.execute_commands(
                box_id="",
                command="ls -l",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_live_view_url(self, client: GboxClient) -> None:
        box = client.v1.boxes.live_view_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(BoxLiveViewURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_live_view_url_with_all_params(self, client: GboxClient) -> None:
        box = client.v1.boxes.live_view_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            expires_in="180m",
        )
        assert_matches_type(BoxLiveViewURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_live_view_url(self, client: GboxClient) -> None:
        response = client.v1.boxes.with_raw_response.live_view_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = response.parse()
        assert_matches_type(BoxLiveViewURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_live_view_url(self, client: GboxClient) -> None:
        with client.v1.boxes.with_streaming_response.live_view_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = response.parse()
            assert_matches_type(BoxLiveViewURLResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_live_view_url(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.with_raw_response.live_view_url(
                box_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_run_code(self, client: GboxClient) -> None:
        box = client.v1.boxes.run_code(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            code='print("Hello, World!")',
        )
        assert_matches_type(BoxRunCodeResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_run_code_with_all_params(self, client: GboxClient) -> None:
        box = client.v1.boxes.run_code(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            code='print("Hello, World!")',
            argv=["--help"],
            envs={
                "PYTHONPATH": "/usr/lib/python",
                "DEBUG": "true",
            },
            language="python",
            api_timeout="30s",
            working_dir="/home/user/scripts",
        )
        assert_matches_type(BoxRunCodeResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_run_code(self, client: GboxClient) -> None:
        response = client.v1.boxes.with_raw_response.run_code(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            code='print("Hello, World!")',
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = response.parse()
        assert_matches_type(BoxRunCodeResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_run_code(self, client: GboxClient) -> None:
        with client.v1.boxes.with_streaming_response.run_code(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            code='print("Hello, World!")',
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = response.parse()
            assert_matches_type(BoxRunCodeResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_run_code(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.with_raw_response.run_code(
                box_id="",
                code='print("Hello, World!")',
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_start(self, client: GboxClient) -> None:
        box = client.v1.boxes.start(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(BoxStartResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_start_with_all_params(self, client: GboxClient) -> None:
        box = client.v1.boxes.start(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            wait=True,
        )
        assert_matches_type(BoxStartResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_start(self, client: GboxClient) -> None:
        response = client.v1.boxes.with_raw_response.start(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = response.parse()
        assert_matches_type(BoxStartResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_start(self, client: GboxClient) -> None:
        with client.v1.boxes.with_streaming_response.start(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = response.parse()
            assert_matches_type(BoxStartResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_start(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.with_raw_response.start(
                box_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_stop(self, client: GboxClient) -> None:
        box = client.v1.boxes.stop(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(BoxStopResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_stop_with_all_params(self, client: GboxClient) -> None:
        box = client.v1.boxes.stop(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            wait=True,
        )
        assert_matches_type(BoxStopResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_stop(self, client: GboxClient) -> None:
        response = client.v1.boxes.with_raw_response.stop(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = response.parse()
        assert_matches_type(BoxStopResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_stop(self, client: GboxClient) -> None:
        with client.v1.boxes.with_streaming_response.stop(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = response.parse()
            assert_matches_type(BoxStopResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_stop(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.with_raw_response.stop(
                box_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_terminate(self, client: GboxClient) -> None:
        box = client.v1.boxes.terminate(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert box is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_terminate_with_all_params(self, client: GboxClient) -> None:
        box = client.v1.boxes.terminate(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            wait=True,
        )
        assert box is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_terminate(self, client: GboxClient) -> None:
        response = client.v1.boxes.with_raw_response.terminate(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = response.parse()
        assert box is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_terminate(self, client: GboxClient) -> None:
        with client.v1.boxes.with_streaming_response.terminate(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = response.parse()
            assert box is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_terminate(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.with_raw_response.terminate(
                box_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_web_terminal_url(self, client: GboxClient) -> None:
        box = client.v1.boxes.web_terminal_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(BoxWebTerminalURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_web_terminal_url_with_all_params(self, client: GboxClient) -> None:
        box = client.v1.boxes.web_terminal_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            expires_in="180m",
        )
        assert_matches_type(BoxWebTerminalURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_web_terminal_url(self, client: GboxClient) -> None:
        response = client.v1.boxes.with_raw_response.web_terminal_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = response.parse()
        assert_matches_type(BoxWebTerminalURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_web_terminal_url(self, client: GboxClient) -> None:
        with client.v1.boxes.with_streaming_response.web_terminal_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = response.parse()
            assert_matches_type(BoxWebTerminalURLResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_web_terminal_url(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.with_raw_response.web_terminal_url(
                box_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_websocket_url(self, client: GboxClient) -> None:
        box = client.v1.boxes.websocket_url(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(BoxWebsocketURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_websocket_url(self, client: GboxClient) -> None:
        response = client.v1.boxes.with_raw_response.websocket_url(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = response.parse()
        assert_matches_type(BoxWebsocketURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_websocket_url(self, client: GboxClient) -> None:
        with client.v1.boxes.with_streaming_response.websocket_url(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = response.parse()
            assert_matches_type(BoxWebsocketURLResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_websocket_url(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            client.v1.boxes.with_raw_response.websocket_url(
                "",
            )


class TestAsyncBoxes:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.retrieve(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(BoxRetrieveResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.with_raw_response.retrieve(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = await response.parse()
        assert_matches_type(BoxRetrieveResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.with_streaming_response.retrieve(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = await response.parse()
            assert_matches_type(BoxRetrieveResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.list()
        assert_matches_type(BoxListResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.list(
            device_type="virtual",
            labels={},
            page=1,
            page_size=10,
            status=["running"],
            type=["linux"],
        )
        assert_matches_type(BoxListResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = await response.parse()
        assert_matches_type(BoxListResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = await response.parse()
            assert_matches_type(BoxListResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_android(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.create_android()
        assert_matches_type(AndroidBox, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_android_with_all_params(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.create_android(
            config={
                "device_type": "virtual",
                "envs": {
                    "ANDROID_LOG_TAGS": "*:V",
                    "ADB_TRACE": "all",
                },
                "expires_in": "15m",
                "labels": {
                    "app": "mobile-testing",
                    "version": "v1.0",
                },
            },
            api_timeout="30s",
            wait=True,
        )
        assert_matches_type(AndroidBox, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create_android(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.with_raw_response.create_android()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = await response.parse()
        assert_matches_type(AndroidBox, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create_android(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.with_streaming_response.create_android() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = await response.parse()
            assert_matches_type(AndroidBox, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_linux(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.create_linux()
        assert_matches_type(LinuxBox, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_linux_with_all_params(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.create_linux(
            config={
                "envs": {
                    "DEBUG": "true",
                    "API_URL": "https://api.example.com",
                },
                "expires_in": "60m",
                "labels": {
                    "project": "web-automation",
                    "environment": "testing",
                },
            },
            api_timeout="30s",
            wait=True,
        )
        assert_matches_type(LinuxBox, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create_linux(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.with_raw_response.create_linux()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = await response.parse()
        assert_matches_type(LinuxBox, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create_linux(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.with_streaming_response.create_linux() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = await response.parse()
            assert_matches_type(LinuxBox, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_display(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.display(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(BoxDisplayResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_display(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.with_raw_response.display(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = await response.parse()
        assert_matches_type(BoxDisplayResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_display(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.with_streaming_response.display(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = await response.parse()
            assert_matches_type(BoxDisplayResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_display(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.with_raw_response.display(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_execute_commands(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.execute_commands(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            command="ls -l",
        )
        assert_matches_type(BoxExecuteCommandsResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_execute_commands_with_all_params(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.execute_commands(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            command="ls -l",
            envs={
                "PATH": "/usr/bin:/bin",
                "NODE_ENV": "production",
            },
            api_timeout="30s",
            working_dir="/home/user/projects",
        )
        assert_matches_type(BoxExecuteCommandsResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_execute_commands(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.with_raw_response.execute_commands(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            command="ls -l",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = await response.parse()
        assert_matches_type(BoxExecuteCommandsResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_execute_commands(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.with_streaming_response.execute_commands(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            command="ls -l",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = await response.parse()
            assert_matches_type(BoxExecuteCommandsResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_execute_commands(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.with_raw_response.execute_commands(
                box_id="",
                command="ls -l",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_live_view_url(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.live_view_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(BoxLiveViewURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_live_view_url_with_all_params(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.live_view_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            expires_in="180m",
        )
        assert_matches_type(BoxLiveViewURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_live_view_url(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.with_raw_response.live_view_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = await response.parse()
        assert_matches_type(BoxLiveViewURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_live_view_url(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.with_streaming_response.live_view_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = await response.parse()
            assert_matches_type(BoxLiveViewURLResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_live_view_url(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.with_raw_response.live_view_url(
                box_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_run_code(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.run_code(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            code='print("Hello, World!")',
        )
        assert_matches_type(BoxRunCodeResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_run_code_with_all_params(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.run_code(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            code='print("Hello, World!")',
            argv=["--help"],
            envs={
                "PYTHONPATH": "/usr/lib/python",
                "DEBUG": "true",
            },
            language="python",
            api_timeout="30s",
            working_dir="/home/user/scripts",
        )
        assert_matches_type(BoxRunCodeResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_run_code(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.with_raw_response.run_code(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            code='print("Hello, World!")',
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = await response.parse()
        assert_matches_type(BoxRunCodeResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_run_code(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.with_streaming_response.run_code(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            code='print("Hello, World!")',
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = await response.parse()
            assert_matches_type(BoxRunCodeResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_run_code(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.with_raw_response.run_code(
                box_id="",
                code='print("Hello, World!")',
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_start(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.start(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(BoxStartResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_start_with_all_params(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.start(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            wait=True,
        )
        assert_matches_type(BoxStartResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_start(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.with_raw_response.start(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = await response.parse()
        assert_matches_type(BoxStartResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_start(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.with_streaming_response.start(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = await response.parse()
            assert_matches_type(BoxStartResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_start(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.with_raw_response.start(
                box_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_stop(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.stop(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(BoxStopResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_stop_with_all_params(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.stop(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            wait=True,
        )
        assert_matches_type(BoxStopResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_stop(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.with_raw_response.stop(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = await response.parse()
        assert_matches_type(BoxStopResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_stop(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.with_streaming_response.stop(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = await response.parse()
            assert_matches_type(BoxStopResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_stop(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.with_raw_response.stop(
                box_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_terminate(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.terminate(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert box is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_terminate_with_all_params(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.terminate(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            wait=True,
        )
        assert box is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_terminate(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.with_raw_response.terminate(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = await response.parse()
        assert box is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_terminate(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.with_streaming_response.terminate(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = await response.parse()
            assert box is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_terminate(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.with_raw_response.terminate(
                box_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_web_terminal_url(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.web_terminal_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(BoxWebTerminalURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_web_terminal_url_with_all_params(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.web_terminal_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            expires_in="180m",
        )
        assert_matches_type(BoxWebTerminalURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_web_terminal_url(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.with_raw_response.web_terminal_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = await response.parse()
        assert_matches_type(BoxWebTerminalURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_web_terminal_url(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.with_streaming_response.web_terminal_url(
            box_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = await response.parse()
            assert_matches_type(BoxWebTerminalURLResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_web_terminal_url(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.with_raw_response.web_terminal_url(
                box_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_websocket_url(self, async_client: AsyncGboxClient) -> None:
        box = await async_client.v1.boxes.websocket_url(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(BoxWebsocketURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_websocket_url(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.boxes.with_raw_response.websocket_url(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        box = await response.parse()
        assert_matches_type(BoxWebsocketURLResponse, box, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_websocket_url(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.boxes.with_streaming_response.websocket_url(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            box = await response.parse()
            assert_matches_type(BoxWebsocketURLResponse, box, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_websocket_url(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `box_id` but received ''"):
            await async_client.v1.boxes.with_raw_response.websocket_url(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ActionExtractParams"]


class ActionExtractParams(TypedDict, total=False):
    instruction: Required[str]
    """The instruction of the action to extract data from the UI interface"""

    schema: object
    """JSON Schema defining the structure of data to extract.

    Supports object, array, string, number, boolean types with validation rules.

    Common use cases:

    - Extract text content: { "type": "string" }
    - Extract structured data: { "type": "object", "properties": {...} }
    - Extract lists: { "type": "array", "items": {...} }
    - Extract with validation: Add constraints like "required", "enum", "pattern",
      etc.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel
from .media_album import MediaAlbum

__all__ = ["MediaListAlbumsResponse"]


class MediaListAlbumsResponse(BaseModel):
    data: List[MediaAlbum]
    """List of albums"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._types import FileTypes, SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["MediaUpdateAlbumParams"]


class MediaUpdateAlbumParams(TypedDict, total=False):
    box_id: Required[Annotated[str, PropertyInfo(alias="boxId")]]

    media: Required[SequenceNotStr[FileTypes]]
    """Media files to add to the album (max size: 512MB per file)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from gbox_sdk import GboxClient, AsyncGboxClient
from tests.utils import assert_matches_type
from gbox_sdk.types.v1 import DeviceInfo, GetDeviceListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDevices:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: GboxClient) -> None:
        device = client.v1.devices.list()
        assert_matches_type(GetDeviceListResponse, device, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: GboxClient) -> None:
        response = client.v1.devices.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = response.parse()
        assert_matches_type(GetDeviceListResponse, device, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: GboxClient) -> None:
        with client.v1.devices.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = response.parse()
            assert_matches_type(GetDeviceListResponse, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get(self, client: GboxClient) -> None:
        device = client.v1.devices.get(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(DeviceInfo, device, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get(self, client: GboxClient) -> None:
        response = client.v1.devices.with_raw_response.get(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = response.parse()
        assert_matches_type(DeviceInfo, device, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get(self, client: GboxClient) -> None:
        with client.v1.devices.with_streaming_response.get(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = response.parse()
            assert_matches_type(DeviceInfo, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.v1.devices.with_raw_response.get(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_to_box(self, client: GboxClient) -> None:
        device = client.v1.devices.to_box(
            device_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(str, device, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_to_box_with_all_params(self, client: GboxClient) -> None:
        device = client.v1.devices.to_box(
            device_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            force=True,
        )
        assert_matches_type(str, device, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_to_box(self, client: GboxClient) -> None:
        response = client.v1.devices.with_raw_response.to_box(
            device_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = response.parse()
        assert_matches_type(str, device, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_to_box(self, client: GboxClient) -> None:
        with client.v1.devices.with_streaming_response.to_box(
            device_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = response.parse()
            assert_matches_type(str, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_to_box(self, client: GboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            client.v1.devices.with_raw_response.to_box(
                device_id="",
            )


class TestAsyncDevices:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncGboxClient) -> None:
        device = await async_client.v1.devices.list()
        assert_matches_type(GetDeviceListResponse, device, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.devices.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = await response.parse()
        assert_matches_type(GetDeviceListResponse, device, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.devices.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = await response.parse()
            assert_matches_type(GetDeviceListResponse, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get(self, async_client: AsyncGboxClient) -> None:
        device = await async_client.v1.devices.get(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(DeviceInfo, device, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.devices.with_raw_response.get(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = await response.parse()
        assert_matches_type(DeviceInfo, device, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.devices.with_streaming_response.get(
            "c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = await response.parse()
            assert_matches_type(DeviceInfo, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.v1.devices.with_raw_response.get(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_to_box(self, async_client: AsyncGboxClient) -> None:
        device = await async_client.v1.devices.to_box(
            device_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )
        assert_matches_type(str, device, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_to_box_with_all_params(self, async_client: AsyncGboxClient) -> None:
        device = await async_client.v1.devices.to_box(
            device_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
            force=True,
        )
        assert_matches_type(str, device, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_to_box(self, async_client: AsyncGboxClient) -> None:
        response = await async_client.v1.devices.with_raw_response.to_box(
            device_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        device = await response.parse()
        assert_matches_type(str, device, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_to_box(self, async_client: AsyncGboxClient) -> None:
        async with async_client.v1.devices.with_streaming_response.to_box(
            device_id="c9bdc193-b54b-4ddb-a035-5ac0c598d32d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            device = await response.parse()
            assert_matches_type(str, device, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_to_box(self, async_client: AsyncGboxClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `device_id` but received ''"):
            await async_client.v1.devices.with_raw_response.to_box(
                device_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from .dir import Dir
from .file import File

__all__ = ["FRenameResponse"]

FRenameResponse: TypeAlias = Union[File, Dir]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .fs import (
    FsResource,
    AsyncFsResource,
    FsResourceWithRawResponse,
    AsyncFsResourceWithRawResponse,
    FsResourceWithStreamingResponse,
    AsyncFsResourceWithStreamingResponse,
)
from .boxes import (
    BoxesResource,
    AsyncBoxesResource,
    BoxesResourceWithRawResponse,
    AsyncBoxesResourceWithRawResponse,
    BoxesResourceWithStreamingResponse,
    AsyncBoxesResourceWithStreamingResponse,
)
from .media import (
    MediaResource,
    AsyncMediaResource,
    MediaResourceWithRawResponse,
    AsyncMediaResourceWithRawResponse,
    MediaResourceWithStreamingResponse,
    AsyncMediaResourceWithStreamingResponse,
)
from .proxy import (
    ProxyResource,
    AsyncProxyResource,
    ProxyResourceWithRawResponse,
    AsyncProxyResourceWithRawResponse,
    ProxyResourceWithStreamingResponse,
    AsyncProxyResourceWithStreamingResponse,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .android import (
    AndroidResource,
    AsyncAndroidResource,
    AndroidResourceWithRawResponse,
    AsyncAndroidResourceWithRawResponse,
    AndroidResourceWithStreamingResponse,
    AsyncAndroidResourceWithStreamingResponse,
)
from .browser import (
    BrowserResource,
    AsyncBrowserResource,
    BrowserResourceWithRawResponse,
    AsyncBrowserResourceWithRawResponse,
    BrowserResourceWithStreamingResponse,
    AsyncBrowserResourceWithStreamingResponse,
)
from .storage import (
    StorageResource,
    AsyncStorageResource,
    StorageResourceWithRawResponse,
    AsyncStorageResourceWithRawResponse,
    StorageResourceWithStreamingResponse,
    AsyncStorageResourceWithStreamingResponse,
)

__all__ = [
    "StorageResource",
    "AsyncStorageResource",
    "StorageResourceWithRawResponse",
    "AsyncStorageResourceWithRawResponse",
    "StorageResourceWithStreamingResponse",
    "AsyncStorageResourceWithStreamingResponse",
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "ProxyResource",
    "AsyncProxyResource",
    "ProxyResourceWithRawResponse",
    "AsyncProxyResourceWithRawResponse",
    "ProxyResourceWithStreamingResponse",
    "AsyncProxyResourceWithStreamingResponse",
    "MediaResource",
    "AsyncMediaResource",
    "MediaResourceWithRawResponse",
    "AsyncMediaResourceWithRawResponse",
    "MediaResourceWithStreamingResponse",
    "AsyncMediaResourceWithStreamingResponse",
    "FsResource",
    "AsyncFsResource",
    "FsResourceWithRawResponse",
    "AsyncFsResourceWithRawResponse",
    "FsResourceWithStreamingResponse",
    "AsyncFsResourceWithStreamingResponse",
    "BrowserResource",
    "AsyncBrowserResource",
    "BrowserResourceWithRawResponse",
    "AsyncBrowserResourceWithRawResponse",
    "BrowserResourceWithStreamingResponse",
    "AsyncBrowserResourceWithStreamingResponse",
    "AndroidResource",
    "AsyncAndroidResource",
    "AndroidResourceWithRawResponse",
    "AsyncAndroidResourceWithRawResponse",
    "AndroidResourceWithStreamingResponse",
    "AsyncAndroidResourceWithStreamingResponse",
    "BoxesResource",
    "AsyncBoxesResource",
    "BoxesResourceWithRawResponse",
    "AsyncBoxesResourceWithRawResponse",
    "BoxesResourceWithStreamingResponse",
    "AsyncBoxesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import TypeAlias

from ...._models import BaseModel
from .action_screenshot_options import ActionScreenshotOptions

__all__ = ["ActionCommonOptions", "Screenshot"]

Screenshot: TypeAlias = Union[ActionScreenshotOptions, bool]


class ActionCommonOptions(BaseModel):
    screenshot: Optional[Screenshot] = None
    """Screenshot options.

    Can be a boolean to enable/disable screenshots, or an object to configure
    screenshot options.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Mapping, cast

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven, FileTypes, SequenceNotStr
from ...._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.boxes import media_create_album_params, media_update_album_params
from ....types.v1.boxes.media_album import MediaAlbum
from ....types.v1.boxes.media_get_media_response import MediaGetMediaResponse
from ....types.v1.boxes.media_list_media_response import MediaListMediaResponse
from ....types.v1.boxes.media_list_albums_response import MediaListAlbumsResponse
from ....types.v1.boxes.media_get_media_support_response import MediaGetMediaSupportResponse

__all__ = ["MediaResource", "AsyncMediaResource"]


class MediaResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MediaResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return MediaResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MediaResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return MediaResourceWithStreamingResponse(self)

    def create_album(
        self,
        box_id: str,
        *,
        name: str,
        media: SequenceNotStr[FileTypes] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MediaAlbum:
        """
        Create a new album with media files

        Args:
          name: Name of the album to create

          media: Media files to include in the album (max size: 512MB per file)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        body = deepcopy_minimal(
            {
                "name": name,
                "media": media,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["media", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            f"/boxes/{box_id}/media/albums",
            body=maybe_transform(body, media_create_album_params.MediaCreateAlbumParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=MediaAlbum,
        )

    def delete_album(
        self,
        album_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Delete an album and all its media files

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not album_name:
            raise ValueError(f"Expected a non-empty value for `album_name` but received {album_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/boxes/{box_id}/media/albums/{album_name}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=NoneType,
        )

    def delete_media(
        self,
        media_name: str,
        *,
        box_id: str,
        album_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Delete a specific media file from an album

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not album_name:
            raise ValueError(f"Expected a non-empty value for `album_name` but received {album_name!r}")
        if not media_name:
            raise ValueError(f"Expected a non-empty value for `media_name` but received {media_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/boxes/{box_id}/media/albums/{album_name}/media/{media_name}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=NoneType,
        )

    def download_media(
        self,
        media_name: str,
        *,
        box_id: str,
        album_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BinaryAPIResponse:
        """
        Download a specific media file from an album

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not album_name:
            raise ValueError(f"Expected a non-empty value for `album_name` but received {album_name!r}")
        if not media_name:
            raise ValueError(f"Expected a non-empty value for `media_name` but received {media_name!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return self._get(
            f"/boxes/{box_id}/media/albums/{album_name}/media/{media_name}/download",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=BinaryAPIResponse,
        )

    def get_album_detail(
        self,
        album_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MediaAlbum:
        """
        Get detailed information about a specific album including its media files

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not album_name:
            raise ValueError(f"Expected a non-empty value for `album_name` but received {album_name!r}")
        return self._get(
            f"/boxes/{box_id}/media/albums/{album_name}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=MediaAlbum,
        )

    def get_media(
        self,
        media_name: str,
        *,
        box_id: str,
        album_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MediaGetMediaResponse:
        """
        Get detailed information about a specific media file

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not album_name:
            raise ValueError(f"Expected a non-empty value for `album_name` but received {album_name!r}")
        if not media_name:
            raise ValueError(f"Expected a non-empty value for `media_name` but received {media_name!r}")
        return cast(
            MediaGetMediaResponse,
            self._get(
                f"/boxes/{box_id}/media/albums/{album_name}/media/{media_name}",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                ),
                cast_to=cast(
                    Any, MediaGetMediaResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def get_media_support(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MediaGetMediaSupportResponse:
        """
        Get supported media file extensions for photos and videos

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._get(
            f"/boxes/{box_id}/media/support",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=MediaGetMediaSupportResponse,
        )

    def list_albums(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MediaListAlbumsResponse:
        """
        Get a list of albums in the box

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return self._get(
            f"/boxes/{box_id}/media/albums",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=MediaListAlbumsResponse,
        )

    def list_media(
        self,
        album_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MediaListMediaResponse:
        """
        Get a list of media files in a specific album

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not album_name:
            raise ValueError(f"Expected a non-empty value for `album_name` but received {album_name!r}")
        return self._get(
            f"/boxes/{box_id}/media/albums/{album_name}/media",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=MediaListMediaResponse,
        )

    def update_album(
        self,
        album_name: str,
        *,
        box_id: str,
        media: SequenceNotStr[FileTypes],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MediaAlbum:
        """
        Add media files to an existing album

        Args:
          media: Media files to add to the album (max size: 512MB per file)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not album_name:
            raise ValueError(f"Expected a non-empty value for `album_name` but received {album_name!r}")
        body = deepcopy_minimal({"media": media})
        files = extract_files(cast(Mapping[str, object], body), paths=[["media", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._patch(
            f"/boxes/{box_id}/media/albums/{album_name}",
            body=maybe_transform(body, media_update_album_params.MediaUpdateAlbumParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=MediaAlbum,
        )


class AsyncMediaResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMediaResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#accessing-raw-response-data-eg-headers
        """
        return AsyncMediaResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMediaResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/babelcloud/gbox-sdk-py#with_streaming_response
        """
        return AsyncMediaResourceWithStreamingResponse(self)

    async def create_album(
        self,
        box_id: str,
        *,
        name: str,
        media: SequenceNotStr[FileTypes] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MediaAlbum:
        """
        Create a new album with media files

        Args:
          name: Name of the album to create

          media: Media files to include in the album (max size: 512MB per file)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        body = deepcopy_minimal(
            {
                "name": name,
                "media": media,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["media", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            f"/boxes/{box_id}/media/albums",
            body=await async_maybe_transform(body, media_create_album_params.MediaCreateAlbumParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=MediaAlbum,
        )

    async def delete_album(
        self,
        album_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Delete an album and all its media files

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not album_name:
            raise ValueError(f"Expected a non-empty value for `album_name` but received {album_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/boxes/{box_id}/media/albums/{album_name}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=NoneType,
        )

    async def delete_media(
        self,
        media_name: str,
        *,
        box_id: str,
        album_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Delete a specific media file from an album

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not album_name:
            raise ValueError(f"Expected a non-empty value for `album_name` but received {album_name!r}")
        if not media_name:
            raise ValueError(f"Expected a non-empty value for `media_name` but received {media_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/boxes/{box_id}/media/albums/{album_name}/media/{media_name}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=NoneType,
        )

    async def download_media(
        self,
        media_name: str,
        *,
        box_id: str,
        album_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncBinaryAPIResponse:
        """
        Download a specific media file from an album

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not album_name:
            raise ValueError(f"Expected a non-empty value for `album_name` but received {album_name!r}")
        if not media_name:
            raise ValueError(f"Expected a non-empty value for `media_name` but received {media_name!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return await self._get(
            f"/boxes/{box_id}/media/albums/{album_name}/media/{media_name}/download",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def get_album_detail(
        self,
        album_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MediaAlbum:
        """
        Get detailed information about a specific album including its media files

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not album_name:
            raise ValueError(f"Expected a non-empty value for `album_name` but received {album_name!r}")
        return await self._get(
            f"/boxes/{box_id}/media/albums/{album_name}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=MediaAlbum,
        )

    async def get_media(
        self,
        media_name: str,
        *,
        box_id: str,
        album_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MediaGetMediaResponse:
        """
        Get detailed information about a specific media file

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not album_name:
            raise ValueError(f"Expected a non-empty value for `album_name` but received {album_name!r}")
        if not media_name:
            raise ValueError(f"Expected a non-empty value for `media_name` but received {media_name!r}")
        return cast(
            MediaGetMediaResponse,
            await self._get(
                f"/boxes/{box_id}/media/albums/{album_name}/media/{media_name}",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                ),
                cast_to=cast(
                    Any, MediaGetMediaResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def get_media_support(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MediaGetMediaSupportResponse:
        """
        Get supported media file extensions for photos and videos

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._get(
            f"/boxes/{box_id}/media/support",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=MediaGetMediaSupportResponse,
        )

    async def list_albums(
        self,
        box_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MediaListAlbumsResponse:
        """
        Get a list of albums in the box

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        return await self._get(
            f"/boxes/{box_id}/media/albums",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=MediaListAlbumsResponse,
        )

    async def list_media(
        self,
        album_name: str,
        *,
        box_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MediaListMediaResponse:
        """
        Get a list of media files in a specific album

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not album_name:
            raise ValueError(f"Expected a non-empty value for `album_name` but received {album_name!r}")
        return await self._get(
            f"/boxes/{box_id}/media/albums/{album_name}/media",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=MediaListMediaResponse,
        )

    async def update_album(
        self,
        album_name: str,
        *,
        box_id: str,
        media: SequenceNotStr[FileTypes],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MediaAlbum:
        """
        Add media files to an existing album

        Args:
          media: Media files to add to the album (max size: 512MB per file)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not box_id:
            raise ValueError(f"Expected a non-empty value for `box_id` but received {box_id!r}")
        if not album_name:
            raise ValueError(f"Expected a non-empty value for `album_name` but received {album_name!r}")
        body = deepcopy_minimal({"media": media})
        files = extract_files(cast(Mapping[str, object], body), paths=[["media", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._patch(
            f"/boxes/{box_id}/media/albums/{album_name}",
            body=await async_maybe_transform(body, media_update_album_params.MediaUpdateAlbumParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=MediaAlbum,
        )


class MediaResourceWithRawResponse:
    def __init__(self, media: MediaResource) -> None:
        self._media = media

        self.create_album = to_raw_response_wrapper(
            media.create_album,
        )
        self.delete_album = to_raw_response_wrapper(
            media.delete_album,
        )
        self.delete_media = to_raw_response_wrapper(
            media.delete_media,
        )
        self.download_media = to_custom_raw_response_wrapper(
            media.download_media,
            BinaryAPIResponse,
        )
        self.get_album_detail = to_raw_response_wrapper(
            media.get_album_detail,
        )
        self.get_media = to_raw_response_wrapper(
            media.get_media,
        )
        self.get_media_support = to_raw_response_wrapper(
            media.get_media_support,
        )
        self.list_albums = to_raw_response_wrapper(
            media.list_albums,
        )
        self.list_media = to_raw_response_wrapper(
            media.list_media,
        )
        self.update_album = to_raw_response_wrapper(
            media.update_album,
        )


class AsyncMediaResourceWithRawResponse:
    def __init__(self, media: AsyncMediaResource) -> None:
        self._media = media

        self.create_album = async_to_raw_response_wrapper(
            media.create_album,
        )
        self.delete_album = async_to_raw_response_wrapper(
            media.delete_album,
        )
        self.delete_media = async_to_raw_response_wrapper(
            media.delete_media,
        )
        self.download_media = async_to_custom_raw_response_wrapper(
            media.download_media,
            AsyncBinaryAPIResponse,
        )
        self.get_album_detail = async_to_raw_response_wrapper(
            media.get_album_detail,
        )
        self.get_media = async_to_raw_response_wrapper(
            media.get_media,
        )
        self.get_media_support = async_to_raw_response_wrapper(
            media.get_media_support,
        )
        self.list_albums = async_to_raw_response_wrapper(
            media.list_albums,
        )
        self.list_media = async_to_raw_response_wrapper(
            media.list_media,
        )
        self.update_album = async_to_raw_response_wrapper(
            media.update_album,
        )


class MediaResourceWithStreamingResponse:
    def __init__(self, media: MediaResource) -> None:
        self._media = media

        self.create_album = to_streamed_response_wrapper(
            media.create_album,
        )
        self.delete_album = to_streamed_response_wrapper(
            media.delete_album,
        )
        self.delete_media = to_streamed_response_wrapper(
            media.delete_media,
        )
        self.download_media = to_custom_streamed_response_wrapper(
            media.download_media,
            StreamedBinaryAPIResponse,
        )
        self.get_album_detail = to_streamed_response_wrapper(
            media.get_album_detail,
        )
        self.get_media = to_streamed_response_wrapper(
            media.get_media,
        )
        self.get_media_support = to_streamed_response_wrapper(
            media.get_media_support,
        )
        self.list_albums = to_streamed_response_wrapper(
            media.list_albums,
        )
        self.list_media = to_streamed_response_wrapper(
            media.list_media,
        )
        self.update_album = to_streamed_response_wrapper(
            media.update_album,
        )


class AsyncMediaResourceWithStreamingResponse:
    def __init__(self, media: AsyncMediaResource) -> None:
        self._media = media

        self.create_album = async_to_streamed_response_wrapper(
            media.create_album,
        )
        self.delete_album = async_to_streamed_response_wrapper(
            media.delete_album,
        )
        self.delete_media = async_to_streamed_response_wrapper(
            media.delete_media,
        )
        self.download_media = async_to_custom_streamed_response_wrapper(
            media.download_media,
            AsyncStreamedBinaryAPIResponse,
        )
        self.get_album_detail = async_to_streamed_response_wrapper(
            media.get_album_detail,
        )
        self.get_media = async_to_streamed_response_wrapper(
            media.get_media,
        )
        self.get_media_support = async_to_streamed_response_wrapper(
            media.get_media_support,
        )
        self.list_albums = async_to_streamed_response_wrapper(
            media.list_albums,
        )
        self.list_media = async_to_streamed_response_wrapper(
            media.list_media,
        )
        self.update_album = async_to_streamed_response_wrapper(
            media.update_album,
        )



import os
from typing import cast
from pathlib import Path
from datetime import datetime
from dataclasses import dataclass
from typing_extensions import List, Union, Optional, Protocol

from gbox_sdk._types import NOT_GIVEN, NotGiven, FileTypes
from gbox_sdk._utils import file_from_path
from gbox_sdk._client import GboxClient
from gbox_sdk.types.v1.boxes.media_album import MediaAlbum
from gbox_sdk.types.v1.boxes.media_get_media_response import MediaGetMediaResponse
from gbox_sdk.types.v1.boxes.media_list_media_response import (
    MediaListMediaResponse,
)
from gbox_sdk.types.v1.boxes.media_list_albums_response import MediaListAlbumsResponse
from gbox_sdk.types.v1.boxes.media_get_media_support_response import MediaGetMediaSupportResponse


@dataclass
class DownloadResult:
    """Result of a single media download operation"""

    success: bool
    file_name: str
    file_path: Optional[str] = None
    error: Optional[str] = None


@dataclass
class DownloadSummary:
    """Summary of album download operation"""

    total_files: int
    successful: int
    failed: int
    results: List[DownloadResult]


media_support_cache: Union[MediaGetMediaSupportResponse, None] = None


def get_media_support(client: GboxClient, box_id: str) -> MediaGetMediaSupportResponse:
    """
    Get supported media extensions from the API
    """
    global media_support_cache
    if media_support_cache is None:
        media_support_cache = client.v1.boxes.media.get_media_support(box_id=box_id)
    return media_support_cache


def is_media_file(file_path: str, client: GboxClient, box_id: str) -> bool:
    """
    Check if a file is an image or video based on its extension
    """
    # Extract lower-cased extension with leading dot (e.g., ".jpg")
    _, ext = os.path.splitext(file_path)
    ext = ext.lower()

    media_support = get_media_support(client, box_id)

    # Normalize supported extensions to include a leading dot and be lower-cased
    photo_extensions = [
        e if e.startswith(".") else f".{e}" for e in (ext_str.lower() for ext_str in media_support.photo)
    ]
    video_extensions = [
        e if e.startswith(".") else f".{e}" for e in (ext_str.lower() for ext_str in media_support.video)
    ]

    return ext in photo_extensions or ext in video_extensions


def get_media_files_from_directory(dir_path: str, client: GboxClient, box_id: str) -> List[str]:
    """
    Recursively get all media files from a directory
    """
    media_files: List[str] = []

    try:
        items = os.listdir(dir_path)

        for item in items:
            full_path = os.path.join(dir_path, item)
            stat = os.stat(full_path)

            if stat.st_mode & 0o40000:  # Check if directory
                # Recursively process subdirectories
                sub_files = get_media_files_from_directory(full_path, client, box_id)
                media_files.extend(sub_files)
            elif stat.st_mode & 0o100000:  # Check if regular file
                # Only include image and video files
                if is_media_file(full_path, client, box_id):
                    media_files.append(full_path)
    except OSError as error:
        raise OSError(f"Error reading directory {dir_path}: {error}") from error

    return media_files


def process_media_item(item: str, client: GboxClient, box_id: str) -> List[FileTypes]:
    """
    Process a media item and return a list of FileTypes
    """
    processed_media: List[FileTypes] = []

    # Handle file:// protocol or local file paths
    if item.startswith("file://"):
        from urllib.parse import unquote, urlparse

        parsed = urlparse(item)
        file_path = unquote(parsed.path)
    else:
        file_path = item

    # Check if path exists
    if not os.path.exists(file_path):
        raise FileNotFoundError(f"Path {file_path} does not exist")

    # Get file stats
    stat = os.stat(file_path)

    if stat.st_mode & 0o40000:  # Check if directory
        # If it's a directory, get all media files from it
        media_files = get_media_files_from_directory(file_path, client, box_id)
        for media_file in media_files:
            processed_media.append(file_from_path(media_file))
    elif stat.st_mode & 0o100000:  # Check if regular file
        # If it's a file, check if it's a media file
        if is_media_file(file_path, client, box_id):
            processed_media.append(file_from_path(file_path))
        else:
            raise ValueError(f"File {file_path} is not a supported image or video file")
    else:
        raise ValueError(f"Path {file_path} is neither a file nor a directory")

    return processed_media


def process_media_array(media: List[Union[FileTypes, str]], client: GboxClient, box_id: str) -> List[FileTypes]:
    processed: List[FileTypes] = []

    for item in media:
        if isinstance(item, str):
            processed.extend(process_media_item(item, client, box_id))
        else:
            processed.append(item)

    return processed


class MediaItemOperator:
    def __init__(
        self,
        client: GboxClient,
        box_id: str,
        album_name: str,
        data: MediaGetMediaResponse,
    ):
        self.client = client
        self.box_id = box_id
        self.album_name = album_name
        self.data = data

    def get_info(self) -> MediaGetMediaResponse:
        """
        Get the info of the media item.

        Example:
            >>> media.get_info()
        """
        return self.client.v1.boxes.media.get_media(
            box_id=self.box_id,
            album_name=self.album_name,
            media_name=self.data.name,
        )

    def download(self, file_path: Optional[str] = None) -> bytes:
        """
        Download media file and optionally save to file system.

        Args:
            file_path: Optional file path to save the downloaded media.
                      If provided, the media will be saved to the specified path.
                      If not provided, only the binary data is returned.

        Returns:
            bytes: The binary data of the downloaded media.

        Example:
            # Return binary data only
            blob = media.download()

            # Download and save to file, then return blob
            blob = media.download("path/to/your/file")
        """
        res = self.client.v1.boxes.media.download_media(
            media_name=self.data.name,
            box_id=self.box_id,
            album_name=self.album_name,
        )

        # Get binary data from response
        binary_data = res.read()

        # If file path is provided, write to file system
        if file_path:
            # Create directory if it doesn't exist
            dir_path = os.path.dirname(file_path)
            if dir_path and not os.path.exists(dir_path):
                os.makedirs(dir_path, exist_ok=True)

            # Write binary data to file
            with open(file_path, "wb") as f:
                f.write(binary_data)

        return binary_data


class MediaAlbumOperator:
    class AlbumInfoLike(Protocol):
        last_modified: datetime
        media_count: float
        name: str
        path: str

    def __init__(self, client: GboxClient, box_id: str, data: "MediaAlbumOperator.AlbumInfoLike"):
        self.client = client
        self.box_id = box_id
        self.data = data

    def list_media_info(self) -> MediaListMediaResponse:
        """
        List all media info in the album.
        """
        return self.client.v1.boxes.media.list_media(box_id=self.box_id, album_name=self.data.name)

    def list_media(self) -> List[MediaItemOperator]:
        """
        List all media in the album.

        Returns:
            List[MediaItemOperator]: A list of MediaItemOperator instances for each media item.

        Example:
            >>> media_items = album.list_media()
        """
        res = self.list_media_info()
        return [MediaItemOperator(self.client, self.box_id, self.data.name, media) for media in res.data]

    def download(self, local_path: str) -> DownloadSummary:
        """
        Download all media files from the album to the specified local folder

        Args:
            local_path: Local directory path to save the downloaded media files

        Returns:
            DownloadSummary: Summary of the download operation including success/failure counts

        Example:
            >>> album.download("path/to/your/folder")
        """
        # Ensure the target directory exists
        Path(local_path).mkdir(parents=True, exist_ok=True)

        # Get all media files in the album
        media_list = self.list_media()

        # Download each media file
        results: List[DownloadResult] = []
        for media_item in media_list:
            try:
                # Get media info to get the file name
                media_info = media_item.get_info()
                file_name = media_info.name
                file_path = os.path.join(local_path, file_name)

                # Download the media file
                media_item.download(file_path)
                results.append(DownloadResult(success=True, file_name=file_name, file_path=file_path))
            except Exception as error:
                results.append(
                    DownloadResult(
                        success=False,
                        file_name=getattr(media_item.data, "name", "unknown"),
                        error=str(error),
                    )
                )

        # Calculate summary statistics
        successful = len([r for r in results if r.success])
        failed = len([r for r in results if not r.success])

        return DownloadSummary(
            total_files=len(results),
            successful=successful,
            failed=failed,
            results=results,
        )

    def append_media(self, media: List[Union[FileTypes, str]]) -> "MediaAlbumOperator":
        """
        Append one or more media files to this album.

        Examples:
            >>> album.append_media(["/path/to/file.jpg"])  # from local file path
            >>> album.append_media(["file:///path/to/file.jpg"])  # from file URL
            >>> album.append_media(["/path/to/folder"])  # add all files from folder recursively
        """
        processed_media = process_media_array(media, self.client, self.box_id)
        self.client.v1.boxes.media.update_album(
            self.data.name,
            box_id=self.box_id,
            media=processed_media,
        )
        self._sync_data()
        return self

    def delete_media(self, media_name: str) -> None:
        """
        Delete a media file from the album.

        Examples:
            >>> album.delete_media("My Media")
        """
        self.client.v1.boxes.media.delete_media(
            box_id=self.box_id,
            album_name=self.data.name,
            media_name=media_name,
        )
        self._sync_data()

    def _sync_data(self) -> None:
        res = self.client.v1.boxes.media.get_album_detail(box_id=self.box_id, album_name=self.data.name)
        self.data = res


class MediaOperator:
    """
    Operator class for managing media operations within a box.
    Provides an interface to interact with and control media operations
    within a box.
    """

    def __init__(self, client: GboxClient, box_id: str):
        """
        Initialize a MediaOperator instance.

        Args:
            client (GboxClient): The Gbox client used for API communication.
            box_id (str): The ID of the box to operate on.
        """
        self.client = client
        self.box_id = box_id

    def list_albums(self) -> List[MediaAlbumOperator]:
        """
        List all albums in the box.

        Examples:
            >>> albums = box.media.list_albums()
        """
        res = self.client.v1.boxes.media.list_albums(box_id=self.box_id)
        return [MediaAlbumOperator(self.client, self.box_id, album) for album in res.data]

    def list_albums_info(self) -> MediaListAlbumsResponse:
        """
        Get a list of albums in the box.

        Examples:
            >>> albums = box.media.list_albums_info()
        """
        return self.client.v1.boxes.media.list_albums(box_id=self.box_id)

    def create_album(
        self,
        *,
        name: str,
        media: Union[List[Union[FileTypes, str]], NotGiven] = NOT_GIVEN,
    ) -> MediaAlbumOperator:
        """
        Create a new album in the box.

        Args:
          name: Name of the album to create

          media: Media files to include in the album (max size: 512MB per file)

        Examples:
            >>> album = box.media.create_album("My Album")
        """
        if media is NOT_GIVEN:
            processed_media: Union[List[FileTypes], NotGiven] = NOT_GIVEN
        else:
            media_list = cast(List[Union[FileTypes, str]], media)
            processed_media = process_media_array(media_list, self.client, self.box_id)
        res = self.client.v1.boxes.media.create_album(
            box_id=self.box_id,
            name=name,
            media=processed_media,
        )
        return MediaAlbumOperator(self.client, self.box_id, res)

    def delete_album(self, album_name: str) -> None:
        """
        Delete an album from the box.

        Examples:
            >>> box.media.delete_album("My Album")
        """
        self.client.v1.boxes.media.delete_album(box_id=self.box_id, album_name=album_name)

    def get_album_info(self, album_name: str) -> MediaAlbum:
        """
        Get the info of an album.

        Examples:
            >>> album = box.media.get_album_info("My Album")
        """
        return self.client.v1.boxes.media.get_album_detail(box_id=self.box_id, album_name=album_name)

    def get_album(self, album_name: str) -> MediaAlbumOperator:
        """
        Get an album.

        Examples:
            >>> album = box.media.get_album("My Album")
        """
        res = self.get_album_info(album_name)
        return MediaAlbumOperator(self.client, self.box_id, res)

    def get_media_support(self) -> MediaGetMediaSupportResponse:
        """
        Get the media support of the box.

        Examples:
            >>> support = box.media.get_media_support()
        """
        return self.client.v1.boxes.media.get_media_support(box_id=self.box_id)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel
from .android_pkg import AndroidPkg

__all__ = ["AndroidListPkgResponse"]


class AndroidListPkgResponse(BaseModel):
    data: List[AndroidPkg]
    """Android pkg list"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ActionRecordingStartParams"]


class ActionRecordingStartParams(TypedDict, total=False):
    duration: str
    """Duration of the recording.

    Default is 30m, max is 30m. The recording will automatically stop when the
    duration time is reached.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Maximum allowed: 30m
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from ...._models import BaseModel

__all__ = ["FExistsResponse", "ExistsFileResult", "NotExistsFileResult"]


class ExistsFileResult(BaseModel):
    exists: bool
    """Exists"""

    type: str
    """Type"""


class NotExistsFileResult(BaseModel):
    exists: bool
    """Exists"""


FExistsResponse: TypeAlias = Union[ExistsFileResult, NotExistsFileResult]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["ProxySetParams", "Auth"]


class ProxySetParams(TypedDict, total=False):
    host: Required[str]
    """The host address of the proxy server"""

    port: Required[float]
    """The port number of the proxy server"""

    auth: Auth
    """Box Proxy Auth"""

    excludes: SequenceNotStr[str]
    """List of IP addresses and domains that should bypass the proxy.

    These addresses will be accessed directly without going through the proxy
    server. Default is ['127.0.0.1', 'localhost']
    """

    pac_url: Annotated[str, PropertyInfo(alias="pacUrl")]
    """PAC (Proxy Auto-Configuration) URL."""


class Auth(TypedDict, total=False):
    password: Required[str]
    """Password for the proxy"""

    username: Required[str]
    """Username for the proxy"""
